#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3dma.h>
#include <cyu3error.h>
#include <cyu3usb.h>
#include <cyu3usbconst.h>
#include <cyu3uart.h>
#include <cyu3utils.h>
#include <cyu3gpif.h>
#include <cyu3pib.h>
#include <cyu3gpio.h>
#include <cyu3mbox.h>
#include "gpif_admux.cydsn/cyfxgpif2config.h"
#include "cyfxmscdemo.h"

static CyU3PThread MscAppThread;
static CyU3PMbox cmdMbox;
static CyU3PDmaChannel glGPIF2Cpu;
static CyU3PDmaChannel glCpu2GPIF;
static uint32_t glCmdBuffer[8] __attribute__ ((aligned (32)));
static CyU3PDmaBuffer_t glDmaCmdBuffer = {(uint8_t *)glCmdBuffer, 32, 32, 0};

uvint32_t *crcdata_reg_value      = (uvint32_t*) 0xE0014128;
uint32_t crc = 0;

void
MscAppThread_Entry (uint32_t input);

static void
CyFxAppMboxCallback (CyBool_t isNewMsg);

void
CyFxAppErrorHandler (CyU3PReturnStatus_t apiRetStatus);

int
main (void)
{
    CyU3PReturnStatus_t status;
    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PSysClockConfig_t clkCfg;

    /* setSysClk400 clock configurations */
    clkCfg.setSysClk400  = CyTrue;   /* FX3 device's master clock is set to a frequency > 400 MHz */
    clkCfg.cpuClkDiv     = 2;        /* CPU clock divider */
    clkCfg.dmaClkDiv     = 2;        /* DMA clock divider */
    clkCfg.mmioClkDiv    = 2;        /* MMIO clock divider */
    clkCfg.useStandbyClk = CyFalse;  /* device has no 32KHz clock supplied */
    clkCfg.clkSrc = CY_U3P_SYS_CLK;  /* Clock source for a peripheral block  */

    /* Initialize the device */
    status = CyU3PDeviceInit (&clkCfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Initialize the caches. Enable only the instruction cache. */
    status = CyU3PDeviceCacheControl (CyTrue, CyFalse, CyFalse);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Configure the IO matrix */
    CyU3PMemSet((uint8_t *)&io_cfg,0,sizeof(io_cfg));
    io_cfg.isDQ32Bit = CyTrue;
    io_cfg.useUart   = CyTrue;
    io_cfg.useI2C    = CyFalse;
    io_cfg.useI2S    = CyFalse;
    io_cfg.useSpi    = CyFalse;
    io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_DEFAULT;
    io_cfg.gpioSimpleEn[0]  = 0;
    io_cfg.gpioSimpleEn[1]  = 0;
    io_cfg.gpioComplexEn[0] = 0;
    io_cfg.gpioComplexEn[1] = 0;
    status = CyU3PDeviceConfigureIOMatrix (&io_cfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:
    /* Cannot recover from this error. */
    while (1);
}

/*
 * Application define function which creates the threads. This is called from
 * the tx_application _define function.
 */
void
CyFxApplicationDefine (
        void)
{
    void *ptr;
    uint32_t retThrdCreate;

    /* Allocate the memory for the threads */
    ptr = CyU3PMemAlloc (CY_FX_MSC_THREAD_STACK);

    /* Create the thread for the application */
    retThrdCreate = CyU3PThreadCreate (&MscAppThread,   /* MSC App Thread structure */
                          "25:MSC Application",         /* Thread ID and Thread name */
                          MscAppThread_Entry,           /* MSC App Thread Entry function */
                          0,                            /* No input parameter to thread */
                          ptr,                          /* Pointer to the allocated thread stack */
                          CY_FX_MSC_THREAD_STACK,       /* MSC App Thread stack size */
                          CY_FX_MSC_THREAD_PRIORITY,    /* MSC App Thread priority */
                          CY_FX_MSC_THREAD_PRIORITY,    /* MSC App Thread priority */
                          CYU3P_NO_TIME_SLICE,          /* No time slice for the application thread */
                          CYU3P_AUTO_START              /* Start the Thread immediately */
                          );

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        /* Thread Creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }
}


/*
 * Entry function for the MscAppThread
 */
void
MscAppThread_Entry (uint32_t input)
{
    CyU3PReturnStatus_t apiRetStatus;
    CyU3PPibClock_t pibClock;
    CyU3PDmaChannelConfig_t dmaConfig;

    /* Initialize the p-port block at 100mhz. Enable DLL for Master mode. */
    pibClock.clkDiv      = 4;
    pibClock.clkSrc      = CY_U3P_SYS_CLK;
    pibClock.isHalfDiv   = CyFalse;
    pibClock.isDllEnable = CyFalse;
    apiRetStatus = CyU3PPibInit (CyTrue, &pibClock);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }

    dmaConfig.size 				= 1024;
    dmaConfig.count 			= 1;
    dmaConfig.prodSckId 		= CY_U3P_PIB_SOCKET_0;
    dmaConfig.consSckId 		= CY_U3P_CPU_SOCKET_CONS;
    dmaConfig.dmaMode 			= CY_U3P_DMA_MODE_BYTE;
    dmaConfig.notification		= 0;
    dmaConfig.cb 				= NULL;
    dmaConfig.prodHeader 		= 0;
    dmaConfig.prodFooter 		= 0;
    dmaConfig.consHeader 		= 0;
    dmaConfig.prodAvailCount 	= 0;
    apiRetStatus = CyU3PDmaChannelCreate (&glGPIF2Cpu, CY_U3P_DMA_TYPE_MANUAL_IN, &dmaConfig);
    apiRetStatus = CyU3PDmaChannelReset(&glGPIF2Cpu);


    dmaConfig.size 				= 1024;
    dmaConfig.count 			= 1;
    dmaConfig.prodSckId 		= CY_U3P_CPU_SOCKET_PROD;
    //dmaConfig.consSckId 		= CY_U3P_PIB_SOCKET_8;
    dmaConfig.consSckId 		= CY_U3P_PIB_SOCKET_1;
    dmaConfig.dmaMode 			= CY_U3P_DMA_MODE_BYTE;
    dmaConfig.notification		= 0;
    dmaConfig.cb 				= NULL;
    dmaConfig.prodHeader 		= 0;
    dmaConfig.prodFooter 		= 0;
    dmaConfig.consHeader 		= 0;
    dmaConfig.prodAvailCount 	= 0;
    apiRetStatus = CyU3PDmaChannelCreate (&glCpu2GPIF, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaConfig);
    apiRetStatus = CyU3PDmaChannelReset(&glCpu2GPIF);

    /* Patch CyFxGpifRegValue before loading to GPIF FSM */
    CyFxGpifRegValue[73] = 0x80300000;	// GPIF_CRC_CONFIG ena crc
    CyFxGpifRegValue[74] = 0x0000FFFF;	// GPIF_CRC_DATA initial
    //CyFxGpifRegValue[75] = 0xFE3FFFC1;	// GPIF_BETA_DEASSERT

    apiRetStatus = CyU3PGpifLoad (&CyFxGpifConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set DMA watermark level */
    apiRetStatus = CyU3PGpifSocketConfigure (0, CY_U3P_PIB_SOCKET_0, 5, CyFalse, 14);

    /* Start the GPIF state machine */
    apiRetStatus = CyU3PGpifSMStart (START, ALPHA_START);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Register mailbox callback */
    CyU3PMboxInit(CyFxAppMboxCallback);

    /* Prepare AP to receive 32 bytes */
    cmdMbox.w0 = 32;
    cmdMbox.w1 = 0;
    apiRetStatus = CyU3PMboxWrite(&cmdMbox);

	glCmdBuffer[0] = 0xC0DE1000;
    glCmdBuffer[1] = 0xC0DE1001;
    glCmdBuffer[2] = 0xC0DE1002;
    glCmdBuffer[3] = 0xC0DE1003;
    glCmdBuffer[4] = 0xC0DE1004;
    glCmdBuffer[5] = 0xC0DE1005;
    glCmdBuffer[6] = 0xC0DE1006;
    glCmdBuffer[7] = 0xC0DE1007;

    apiRetStatus = CyU3PDmaChannelSetupSendBuffer(&glCpu2GPIF, &glDmaCmdBuffer);
    if(apiRetStatus != CY_U3P_SUCCESS)
    {
    	CyU3PDeviceReset(CyFalse);
    }
    crc = *crcdata_reg_value;

    while(1);
}



/* MSC application error handler */
void
CyFxAppErrorHandler (CyU3PReturnStatus_t apiRetStatus)
{
    for (;;)
    {
        /* Thread Sleep : 100 ms */
        CyU3PThreadSleep (100);
    }
}


/* Callback for receiving notification of mailbox messages from the external processor. */
static void
CyFxAppMboxCallback (CyBool_t isNewMsg)
{
    /* Only incoming messages are handled here. Outgoing messages are queued directly from the thread. */
    if (isNewMsg)
    {
        /* Read the request into the global buffer and send an event to the thread. */
        CyU3PMboxRead (&cmdMbox);
    }
}
